import fs from 'node:fs/promises';
import { bot } from '../index.mjs';

let _config;
try {
  _config = JSON.parse(await fs.readFile('config.json'));
} catch(err) {
  throw err;
}

export default new class {
  suits = { "♠": "S", "♣": "C", "♦": "D", "♥": "H" };
  stripColors = msg => msg.replace(/\x03\d{0,2}(,\d{0,2}|\x02\x02)?/g, '');
  rand = (max = 1) => ~~(Math.random() * (max - 1) + 1);

  config = {
    get(k) {
      return _config.bot[k];
    },
    async set(k, v) {
      _config.bot[k] = v;
      await this.writeConfig();
      return this.get(k);
    },
    getFull() {
      return _config;
    },
    async writeConfig() {
      try {
        await fs.writeFile('config.json', JSON.stringify(_config, null, 2));
        return true;
      } catch(err) {
        throw err;
      }
    }
  };

  parseCards(msg, output = []) {
    for(const c of this.stripColors(msg).match(/(\w+)([♠♣️♦♥️️])/g))
      output.push(c.length === 3 ? `T${this.suits[c[2]]}` : `${c[0]}${this.suits[c[1]]}`);
    return output;
  };

  async sendMsg(msg) {
    return (await bot.clients.filter(async c => (await c).name == this.config.getFull().clients[0].network)[0])
      .client.sendmsg("normal", this.config.get('channel'), msg);
  };
};
